import configparser
import dataclasses
from enum import Enum
from typing import Dict, List, Literal, Optional, Union

import pydantic
from pydantic import BaseModel


class FieldDataType(pydantic.BaseModel):
    meta: Dict[str, str]
    type: str  # noqa: A003


class FieldDefinition(BaseModel):
    name: str
    description: Optional[str] = None
    primary: bool
    optional: bool
    data_type: FieldDataType


class IcebergTableProperties(pydantic.BaseModel):
    format: Optional[str] = None  # noqa: A003
    partitioning: Optional[List[str]] = None
    location: Optional[str] = None
    format_version: Optional[int] = None


class StreamingSchemaBase(pydantic.BaseModel):
    type: Literal["streaming"]  # noqa: A003
    fields: Optional[List[FieldDefinition]]
    iceberg_table_properties: Optional[IcebergTableProperties] = None


class StoredSchemaBase(pydantic.BaseModel):
    type: Literal["stored"]  # noqa: A003
    fields: Optional[List[FieldDefinition]]


class SubsetSchema(BaseModel):
    type: Literal["subset"]  # noqa: A003
    parent_product: str
    columns: List[str]


class CreateStreamingSchema(StreamingSchemaBase):
    flow_type: str


class CreateSubsetDataProduct(BaseModel):
    name: str
    description: str
    details: SubsetSchema


class CreateStreamingDataProduct(pydantic.BaseModel):
    name: str
    description: str
    details: CreateStreamingSchema


class CreateStoredDataProduct(pydantic.BaseModel):
    name: str
    description: str
    details: StoredSchemaBase


class CreateDataProduct(pydantic.BaseModel):
    name: str
    description: str
    details: Union[StoredSchemaBase, CreateStreamingSchema, SubsetSchema] = pydantic.Field(..., discriminator="type")


class StreamingDataProductSchema(StreamingSchemaBase):
    fields: List[FieldDefinition]


class StoredDataProductSchema(StoredSchemaBase):
    fields: List[FieldDefinition]


class UpdateDataProductSchema(pydantic.BaseModel):
    details: Union[StreamingDataProductSchema, StoredDataProductSchema] = pydantic.Field(..., discriminator="type")


class ExpectationItem(pydantic.BaseModel):
    expectation_type: str
    kwargs: dict
    meta: dict


class ExpectationWeights(pydantic.BaseModel):
    accuracy: float
    completeness: float
    consistency: float
    uniqueness: float
    validity: float


class ExpectationColumnThresholds(BaseModel):
    accuracy: Union[float, None]
    completeness: Union[float, None]
    consistency: Union[float, None]
    uniqueness: Union[float, None]
    validity: Union[float, None]


class ExpectationThresholds(BaseModel):
    table: float
    columns: Dict[str, ExpectationColumnThresholds]


class UpdateQualityExpectations(pydantic.BaseModel):
    custom_details: List[ExpectationItem]
    weights: Union[ExpectationWeights, None]
    thresholds: Union[ExpectationThresholds, None]


class RegisterCore(BaseModel):
    partition: str
    name: str


class RemoveCore(BaseModel):
    urn: str


class CreateSource(pydantic.BaseModel):
    name: str
    description: Union[str, None] = None


class ExternalDatabaseConnectionDetails(pydantic.BaseModel):
    type: Literal["external_database"]  # noqa: A003
    engine: str
    schema_: str = pydantic.Field(..., alias="schema")
    host: str
    port: int
    database: str
    user_env: str
    password_env: str


class FileConnectionDetails(pydantic.BaseModel):
    type: Literal["file"]  # noqa: A003
    url: str
    access_key_env: Optional[str]
    access_secret_env: Optional[str]


class CreateConnection(pydantic.BaseModel):
    name: str
    details: Union[
        ExternalDatabaseConnectionDetails,
        FileConnectionDetails,
    ]


class TableConnectionParams(pydantic.BaseModel):
    type: Literal["table"]  # noqa: A003
    table: str


class QueryConnectionParams(pydantic.BaseModel):
    type: Literal["query"]  # noqa: A003
    query: str


class CSVConnectionParams(pydantic.BaseModel):
    type: Literal["csv"]  # noqa: A003
    path: Union[str, List[str]]
    has_header: Optional[bool]
    delimiter: Optional[str]
    quote_char: Optional[str]
    escape_char: Optional[str]


class CreateSourceConnection(pydantic.BaseModel):
    details: Union[
        TableConnectionParams,
        QueryConnectionParams,
        CSVConnectionParams,
    ]


class Auth(BaseModel):
    access_token: str = ""
    expires_in: Optional[int] = None
    refresh_token: str = ""
    refresh_expires_in: Optional[int] = None


class OptionalProfile(BaseModel):
    gateway_api_url: str = ""
    registry_api_url: str = ""
    iam_api_url: str = ""
    storage_api_url: str = ""
    user: str = ""
    access_token: str = ""
    refresh_token: str = ""
    ignore_tls: bool = False


class Profile(OptionalProfile):
    gateway_api_url: str
    registry_api_url: str
    iam_api_url: str
    storage_api_url: str
    user: str
    access_token: str
    refresh_token: str
    ignore_tls: bool


class EffectEnum(Enum):
    allow = "allow"
    deny = "deny"


class Statement(BaseModel):
    sid: str
    principal: List[str]
    action: List[str]
    resource: List[str]
    condition: Optional[List[str]] = None
    effect: EffectEnum = EffectEnum.allow

    class Config:
        """Pydantic configuration object."""

        use_enum_values = True


class Statements(BaseModel):
    statements: List[Statement]


class Policy(BaseModel):
    version: str = "2022-10-01"
    statements: List[Statement]


class UserPolicy(BaseModel):
    user: str
    policy: Policy


@dataclasses.dataclass
class Common:
    gateway_api_url: str
    registry_api_url: str
    iam_api_url: str
    storage_api_url: str
    profile_name: str
    config: configparser.ConfigParser
    profile: Optional[Union[Profile, OptionalProfile]]

    def get_gateway_api_url(self) -> str:
        """Return gateway api url.

        If a user profile is provided and defines a gateway url, return that,
        otherwise or fall back to cli defined default.
        """
        if self.profile and self.profile.gateway_api_url:
            return self.profile.gateway_api_url
        return self.gateway_api_url

    def get_registry_api_url(self) -> str:
        """Return registry api url.

        If a user profile is provided and defines a registry url, return that,
        otherwise or fall back to cli defined default.
        """
        if self.profile and self.profile.registry_api_url:
            return self.profile.registry_api_url
        return self.registry_api_url

    def get_iam_api_url(self) -> str:
        """Return iam api url.

        If a user profile is provided and defines a iam url, return that,
        otherwise or fall back to cli defined default.
        """
        if self.profile and self.profile.iam_api_url:
            return self.profile.iam_api_url
        return self.iam_api_url

    def get_storage_api_url(self) -> str:
        """Return storage api url.

        If a user profile is provided and defines a storage url, return that,
        otherwise or fall back to cli defined default.
        """
        if self.profile and self.profile.storage_api_url:
            return self.profile.storage_api_url
        return self.storage_api_url



import pathlib

PROFILE_FILENAME = ".neosctl"
PROFILE_FILEPATH = pathlib.Path().home() / PROFILE_FILENAME


DEFAULT_PROFILE = "default"


SUCCESS_CODE = 200
REDIRECT_CODE = 300
BAD_REQUEST_CODE = 400
UNAUTHORISED_CODE = 401



import configparser
import json
import pathlib
import typing

import click
import httpx
import pydantic
import typer
from pygments import formatters, highlight, lexers

from neosctl import constant, schema


def dumps_formatted_json(payload: typing.Union[typing.Dict, typing.List]) -> str:
    """Dump formatted json.

    json.dump provided payload with indent 2 and sorted keys.
    """
    return json.dumps(payload, indent=2, sort_keys=True)


def prettify_json(payload: typing.Union[typing.Dict, typing.List]) -> str:
    """Dump formatted json with colour highlighting."""
    return highlight(dumps_formatted_json(payload), lexers.JsonLexer(), formatters.TerminalFormatter())


def is_success_response(response: httpx.Response) -> bool:
    """Check if a response is `successful`."""
    if constant.SUCCESS_CODE <= response.status_code < constant.REDIRECT_CODE:
        return True
    return False


def exit_with_output(msg: str, exit_code: int = 0) -> typer.Exit:
    """Render output to terminal and exit."""
    typer.echo(msg)

    return typer.Exit(exit_code)


def process_response(
    response: httpx.Response,
    render_callable: typing.Callable[[typing.Union[typing.Dict, typing.List]], str] = prettify_json,
) -> None:
    """Process a server response, render the output and exit."""
    exit_code = 0
    data = response.json()
    if response.status_code >= constant.BAD_REQUEST_CODE:
        exit_code = 1
        message = prettify_json(data)
    else:
        message = render_callable(data)

    raise exit_with_output(
        msg=message,
        exit_code=exit_code,
    )


def read_config_dotfile() -> configparser.ConfigParser:
    """Read in `.neosctl` configuration file and parse."""
    c = configparser.ConfigParser()
    c.read(constant.PROFILE_FILEPATH)
    return c


def get_optional_user_profile_section(
    c: configparser.ConfigParser,
    profile_name: str,
) -> typing.Optional[configparser.SectionProxy]:
    """Get profile from neosctl configuration.

    Returns:
    -------
    Return configparser.SectionProxy if found, or None.
    """
    try:
        return c[profile_name]
    except KeyError:
        return None


def get_user_profile_section(c: configparser.ConfigParser, profile_name: str) -> configparser.SectionProxy:
    """Get profile from neosctl configuration.

    If profile is not found exit cli.

    Returns:
    -------
    configparser.SectionProxy for the requested profile.
    """
    try:
        return c[profile_name]
    except KeyError:
        pass

    raise exit_with_output(
        msg=f"Profile {profile_name} not found.",
        exit_code=1,
    )


def get_user_profile(
    c: configparser.ConfigParser,
    profile_name: str,
    *,
    allow_missing: bool = False,
) -> typing.Optional[typing.Union[schema.Profile, schema.OptionalProfile]]:
    """Get user profile.

    If allow_missing if False and the profile is not found, the cli will exit with an error output.
    """
    if allow_missing:
        profile_config = get_optional_user_profile_section(c, profile_name)
    else:
        profile_config = get_user_profile_section(c, profile_name)

    if profile_config:
        try:
            return schema.Profile(**profile_config)
        except pydantic.ValidationError as e:
            required_fields = [str(err["loc"][0]) for err in e.errors() if err["msg"] == "field required"]
            raise exit_with_output(  # noqa: TRY200, B904
                msg=("Profile dotfile doesn't include fields:\n  {}\nUse neosctl -p {} profile init").format(
                    "\n  ".join(required_fields),
                    profile_name,
                ),
                exit_code=1,
            )
    return None


def bearer(ctx: click.Context) -> typing.Optional[typing.Dict]:
    """Generate bearer authorization header."""
    if not (ctx.obj.profile and ctx.obj.profile.access_token != ""):  # nosec: B105
        return None

    return {"Authorization": f"Bearer {ctx.obj.profile.access_token}"}


def check_profile_exists(ctx: click.Context) -> bool:
    """Check if a profile exists in neosctl configuration or exit."""
    if not ctx.obj.profile:
        raise exit_with_output(
            msg=f"Profile not found! Run neosctl -p {ctx.obj.profile_name} profile init",
            exit_code=1,
        )

    return True


def upsert_config(
    ctx: click.Context,
    profile: schema.Profile,
) -> configparser.ConfigParser:
    """Update neosctl configuration profile in place."""
    ctx.obj.config[ctx.obj.profile_name] = profile.dict()

    with constant.PROFILE_FILEPATH.open("w") as profile_file:
        ctx.obj.config.write(profile_file)

    return ctx.obj.config


def remove_config(
    ctx: click.Context,
) -> configparser.ConfigParser:
    """Remove a profile from neosctl configuration."""
    if not ctx.obj.config.remove_section(ctx.obj.profile_name):
        raise exit_with_output(
            msg=f"Can not remove {ctx.obj.profile_name} profile, profile not found.",
            exit_code=1,
        )

    with constant.PROFILE_FILEPATH.open("w") as profile_file:
        ctx.obj.config.write(profile_file)

    return ctx.obj.config


def get_file_location(filepath: str) -> pathlib.Path:
    """Get a Path for the provided filepath, exit if not found."""
    fp = pathlib.Path(filepath)
    if not fp.exists():
        raise exit_with_output(
            msg=f"Can not find file: {fp}",
            exit_code=1,
        )
    return fp


def load_json_file(fp: pathlib.Path, content_type: str) -> typing.Union[typing.Dict, typing.List]:
    """Load contents of json file, exit if not found."""
    with fp.open() as f:
        try:
            data = json.load(f)
        except json.decoder.JSONDecodeError:
            raise exit_with_output(  # noqa: TRY200, B904
                msg=f"Invalid {content_type} file, must be json format.",
                exit_code=1,
            )
            return []  # never reached as raise exit_with_output, but it makes type checker happy

    return data


def load_fields_file(fp: pathlib.Path, content_type: str = "schema") -> typing.List:
    """Load contents of json file containing list of objects."""
    r = load_json_file(fp, content_type)
    return list(r)


def load_object_file(fp: pathlib.Path, content_type: str) -> typing.Dict:
    """Load contents of json file containing an object."""
    r = load_json_file(fp, content_type)
    return dict(r)


def _request(ctx: click.Context, method: str, url: str, **kwargs: ...) -> httpx.Response:
    return httpx.request(method=method, url=url, headers=bearer(ctx), verify=not ctx.obj.profile.ignore_tls, **kwargs)


def get(ctx: click.Context, url: str, **kwargs: ...) -> httpx.Response:
    """Execute a GET request."""
    return _request(ctx, "GET", url, **kwargs)


def post(ctx: click.Context, url: str, **kwargs: ...) -> httpx.Response:
    """Execute a POST request."""
    return _request(ctx, "POST", url, **kwargs)


def put(ctx: click.Context, url: str, **kwargs: ...) -> httpx.Response:
    """Execute a PUT request."""
    return _request(ctx, "PUT", url, **kwargs)


def patch(ctx: click.Context, url: str, **kwargs: ...) -> httpx.Response:
    """Execute a PATCH request."""
    return _request(ctx, "PATCH", url, **kwargs)


def delete(ctx: click.Context, url: str, **kwargs: ...) -> httpx.Response:
    """Execute a DELETE request."""
    return _request(ctx, "DELETE", url, **kwargs)


def sanitize(
    ctx: click.Context,  # noqa: ARG001
    param: click.Parameter,  # noqa: ARG001
    value: typing.Optional[str],
) -> typing.Optional[str]:
    """Parameter's sanitize callback."""
    if value and isinstance(value, str):
        return value.rstrip("\r\n")

    return value


def validate_string_not_empty(
    ctx: click.Context,
    param: click.Parameter,
    value: str,
) -> str:
    """String validation callback."""
    value = value.strip()

    if not value:
        message = "Value must be a non-empty string."
        raise typer.BadParameter(message, ctx=ctx, param=param)

    return value


def validate_strings_are_not_empty(
    ctx: click.Context,
    param: click.Parameter,
    values: typing.List[str],
) -> typing.List[str]:
    """List of strings validation callback."""
    return [validate_string_not_empty(ctx, param, value) for value in values]



import pathlib
import typing

import httpx
import typer

from neosctl import constant, util
from neosctl.auth import ensure_login
from neosctl.util import process_response

app = typer.Typer()

bucket_app = typer.Typer()
object_app = typer.Typer()
tagging_app = typer.Typer()

app.add_typer(bucket_app, name="bucket", help="Manage object buckets.")
app.add_typer(object_app, name="object", help="Manage objects.")
object_app.add_typer(tagging_app, name="tags", help="Manage object tags.")


def _storage_url(ctx: typer.Context, postfix: str) -> str:
    return "{}/{}".format(ctx.obj.storage_api_url.rstrip("/"), postfix)


def _load_statement(statement: typing.Optional[str], statement_filepath: typing.Optional[str]) -> str:
    if statement is not None:
        return statement

    if statement_filepath is not None:
        fp = util.get_file_location(statement_filepath or "")

        return fp.read_text()

    raise util.exit_with_output(
        msg="At least one of --statement/--statement-filepath is required.",
        exit_code=1,
    )


def _load_params(params_filepath: typing.Optional[str]) -> typing.Union[typing.List[typing.Any], None]:
    if params_filepath:
        fp = util.get_file_location(params_filepath)

        return util.load_fields_file(fp, "params")

    return None


def _handle(
    ctx: typer.Context,
    postfix: str,
    statement: typing.Optional[str] = None,
    statement_filepath: typing.Optional[str] = None,
    params_filepath: typing.Optional[str] = None,
) -> None:
    @ensure_login
    def _request(ctx: typer.Context, params: typing.Union[typing.List[typing.Any], None]) -> httpx.Response:
        return util.post(
            ctx,
            url=_storage_url(ctx, postfix),
            json={
                "statement": statement,
                "params": params,
            },
        )

    statement = _load_statement(statement, statement_filepath)
    params = _load_params(params_filepath)

    r = _request(ctx, params)
    process_response(r)


@app.command()
def execute(
    ctx: typer.Context,
    statement: typing.Optional[str] = typer.Option(
        None,
        "--statement",
        "-s",
        help="pSQL statement",
        callback=util.sanitize,
    ),
    statement_filepath: typing.Optional[str] = typer.Option(
        None,
        "--statement-filepath",
        "-sf",
        help="Filepath for statement sql file.",
        callback=util.sanitize,
    ),
    params_filepath: typing.Optional[str] = typer.Option(
        None,
        "--params-filepath",
        "-pf",
        help="Filepath for statement params json file.",
        callback=util.sanitize,
    ),
) -> None:
    """Execute a statement."""
    _handle(ctx, "execute", statement, statement_filepath, params_filepath)


@app.command()
def executemany(
    ctx: typer.Context,
    statement: typing.Optional[str] = typer.Option(
        None,
        "--statement",
        "-s",
        help="pSQL statement",
        callback=util.sanitize,
    ),
    statement_filepath: typing.Optional[str] = typer.Option(
        None,
        "--statement-filepath",
        "-sf",
        help="Filepath for statement sql file.",
        callback=util.sanitize,
    ),
    params_filepath: typing.Optional[str] = typer.Option(
        None,
        "--params-filepath",
        "-pf",
        help="Filepath for statement params json file.",
        callback=util.sanitize,
    ),
) -> None:
    """Execute a statement with multiple input params."""
    _handle(ctx, "executemany", statement, statement_filepath, params_filepath)


@app.command()
def fetch(
    ctx: typer.Context,
    statement: typing.Optional[str] = typer.Option(
        None,
        "--statement",
        "-s",
        help="pSQL statement",
        callback=util.sanitize,
    ),
    statement_filepath: typing.Optional[str] = typer.Option(
        None,
        "--statement-filepath",
        "-sf",
        help="Filepath for statement sql file.",
        callback=util.sanitize,
    ),
    params_filepath: typing.Optional[str] = typer.Option(
        None,
        "--params-filepath",
        "-pf",
        help="Filepath for statement params json file.",
        callback=util.sanitize,
    ),
) -> None:
    """Fetch results of a statement."""
    _handle(ctx, "fetch", statement, statement_filepath, params_filepath)


@app.command()
def fetchrow(
    ctx: typer.Context,
    statement: typing.Optional[str] = typer.Option(
        None,
        "--statement",
        "-s",
        help="pSQL statement",
        callback=util.sanitize,
    ),
    statement_filepath: typing.Optional[str] = typer.Option(
        None,
        "--statement-filepath",
        "-sf",
        help="Filepath for statement sql file.",
        callback=util.sanitize,
    ),
    params_filepath: typing.Optional[str] = typer.Option(
        None,
        "--params-filepath",
        "-pf",
        help="Filepath for statement params json file.",
        callback=util.sanitize,
    ),
) -> None:
    """Fetch first result of a statement."""
    _handle(ctx, "fetchrow", statement, statement_filepath, params_filepath)


@bucket_app.command(name="create")
def create_bucket(
    ctx: typer.Context,
    bucket_name: str = typer.Argument(
        ...,
        help="Bucket name",
        callback=util.validate_string_not_empty,
    ),
) -> None:
    """Create new bucket."""

    @ensure_login
    def _request(ctx: typer.Context, bucket_name: str) -> httpx.Response:
        return util.put(ctx, url=_storage_url(ctx, f"objects/{bucket_name}"))

    response = _request(ctx, bucket_name)
    process_response(response)


@bucket_app.command(name="list")
def list_buckets(ctx: typer.Context) -> None:
    """List buckets."""

    @ensure_login
    def _request(ctx: typer.Context) -> httpx.Response:
        return util.get(ctx, url=_storage_url(ctx, "objects"))

    response = _request(ctx)
    process_response(response)


@bucket_app.command(name="delete")
def delete_bucket(
    ctx: typer.Context,
    bucket_name: str = typer.Argument(
        ...,
        help="Bucket name",
        callback=util.validate_string_not_empty,
    ),
) -> None:
    """Delete bucket."""

    @ensure_login
    def _request(ctx: typer.Context, bucket_name: str) -> httpx.Response:
        return util.delete(ctx, url=_storage_url(ctx, f"objects/{bucket_name}"))

    response = _request(ctx, bucket_name)
    process_response(response)


@object_app.command(name="create")
def create_object(
    ctx: typer.Context,
    bucket_name: str = typer.Argument(
        ...,
        help="Bucket name",
        callback=util.validate_string_not_empty,
    ),
    object_name: str = typer.Argument(
        ...,
        help="Object name",
        callback=util.validate_string_not_empty,
    ),
    file: str = typer.Argument(
        ...,
        help="Path to the object file.",
        callback=util.validate_string_not_empty,
    ),
) -> None:
    """Create object."""

    @ensure_login
    def _request(ctx: typer.Context, bucket_name: str, object_name: str, file: str) -> httpx.Response:
        with pathlib.Path(file).open("rb") as fh:
            file_content = fh.read()

        return util.put(ctx, url=_storage_url(ctx, f"objects/{bucket_name}/{object_name}"), data=file_content)

    response = _request(ctx, bucket_name, object_name, file)
    process_response(response)


@object_app.command(name="list")
def list_objects(
    ctx: typer.Context,
    bucket_name: str = typer.Argument(
        ...,
        help="Bucket name",
        callback=util.validate_string_not_empty,
    ),
) -> None:
    """List objects."""

    @ensure_login
    def _request(ctx: typer.Context, bucket_name: str) -> httpx.Response:
        return util.get(ctx, url=_storage_url(ctx, f"objects/{bucket_name}"))

    response = _request(ctx, bucket_name)
    process_response(response)


@object_app.command(name="get")
def get_object(
    ctx: typer.Context,
    bucket_name: str = typer.Argument(
        ...,
        help="Bucket name",
        callback=util.validate_string_not_empty,
    ),
    object_name: str = typer.Argument(
        ...,
        help="Object name",
        callback=util.validate_string_not_empty,
    ),
    file: str = typer.Argument(
        ...,
        help="Path to file where to store the object.",
        callback=util.validate_string_not_empty,
    ),
) -> None:
    """Get object."""

    @ensure_login
    def _request(ctx: typer.Context, bucket_name: str, object_name: str) -> httpx.Response:
        return util.get(ctx, url=_storage_url(ctx, f"objects/{bucket_name}/{object_name}"))

    response = _request(ctx, bucket_name, object_name)
    if response.status_code >= constant.BAD_REQUEST_CODE:
        process_response(response)

    with pathlib.Path(file).open("wb") as fh:
        fh.write(response.content)


@object_app.command(name="delete")
def delete_object(
    ctx: typer.Context,
    bucket_name: str = typer.Argument(
        ...,
        help="Bucket name",
        callback=util.validate_string_not_empty,
    ),
    object_name: str = typer.Argument(
        ...,
        help="Object name",
        callback=util.validate_string_not_empty,
    ),
) -> None:
    """Delete object."""

    @ensure_login
    def _request(ctx: typer.Context, bucket_name: str, object_name: str) -> httpx.Response:
        return util.delete(ctx, url=_storage_url(ctx, f"objects/{bucket_name}/{object_name}"))

    response = _request(ctx, bucket_name, object_name)
    process_response(response)


@tagging_app.command(name="set")
def set_object_tags(
    ctx: typer.Context,
    bucket_name: str = typer.Argument(
        ...,
        help="Bucket name",
        callback=util.validate_string_not_empty,
    ),
    object_name: str = typer.Argument(
        ...,
        help="Object name",
        callback=util.validate_string_not_empty,
    ),
    tags: typing.List[str] = typer.Argument(
        ...,
        help="Tags as pairs of key=value",
        callback=util.validate_strings_are_not_empty,
    ),
) -> None:
    """Set object tags. Be aware that this command overwrites any tags that are already set to the object."""

    @ensure_login
    def _request(ctx: typer.Context, bucket_name: str, object_name: str, tags: typing.List[str]) -> httpx.Response:
        tags_list = []
        for tag in tags:
            key, value = tag.split("=", 1)
            tags_list.append({"key": key, "value": value})
        return util.put(
            ctx,
            url=_storage_url(ctx, f"objects/{bucket_name}/{object_name}?tagging"),
            json={"tags": tags_list},
        )

    response = _request(ctx, bucket_name, object_name, tags)
    process_response(response)


@tagging_app.command(name="get")
def get_object_tags(
    ctx: typer.Context,
    bucket_name: str = typer.Argument(
        ...,
        help="Bucket name",
        callback=util.validate_string_not_empty,
    ),
    object_name: str = typer.Argument(
        ...,
        help="Object name",
        callback=util.validate_string_not_empty,
    ),
) -> None:
    """Get object tags."""

    @ensure_login
    def _request(ctx: typer.Context, bucket_name: str, object_name: str) -> httpx.Response:
        return util.get(ctx, url=_storage_url(ctx, f"objects/{bucket_name}/{object_name}?tagging"))

    response = _request(ctx, bucket_name, object_name)
    process_response(response)


@tagging_app.command(name="delete")
def delete_object_tags(
    ctx: typer.Context,
    bucket_name: str = typer.Argument(
        ...,
        help="Bucket name",
        callback=util.validate_string_not_empty,
    ),
    object_name: str = typer.Argument(
        ...,
        help="Object name",
        callback=util.validate_string_not_empty,
    ),
) -> None:
    """Delete object tags."""

    @ensure_login
    def _request(ctx: typer.Context, bucket_name: str, object_name: str) -> httpx.Response:
        return util.delete(ctx, url=_storage_url(ctx, f"objects/{bucket_name}/{object_name}?tagging"))

    response = _request(ctx, bucket_name, object_name)
    process_response(response)



import json
from typing import Optional

import httpx
import typer

from neosctl import schema, util
from neosctl.auth import ensure_login
from neosctl.util import process_response

app = typer.Typer()


def _source_url(ctx: typer.Context) -> str:
    return "{}/source".format(ctx.obj.gateway_api_url.rstrip("/"))


def _connection_url(ctx: typer.Context) -> str:
    return "{}/connection".format(ctx.obj.gateway_api_url.rstrip("/"))


def _source_connection_url(ctx: typer.Context, source_id: int, connection_id: int) -> str:
    return "{url}/source/{source_id}/connection/{connection_id}".format(
        url=ctx.obj.gateway_api_url.rstrip("/"),
        source_id=source_id,
        connection_id=connection_id,
    )


@app.command()
def create_source(
    ctx: typer.Context,
    name: str = typer.Option(..., "--name", "-n", help="Source name", callback=util.sanitize),
    description: str = typer.Option(..., "--description", "-d", help="Source description", callback=util.sanitize),
) -> None:
    """Create a new source."""

    @ensure_login
    def _request(ctx: typer.Context, content: schema.CreateSource) -> httpx.Response:
        return util.post(
            ctx,
            url=_source_url(ctx),
            json=content.dict(exclude_none=True, by_alias=True),
        )

    payload = schema.CreateSource(name=name, description=description)
    r = _request(ctx, payload)
    process_response(r)


@app.command()
def list_sources(
    ctx: typer.Context,
) -> None:
    """List sources."""

    @ensure_login
    def _request(ctx: typer.Context) -> httpx.Response:
        return util.get(
            ctx,
            url=_source_url(ctx),
        )

    r = _request(ctx)
    process_response(r)


@app.command()
def get_source(
    ctx: typer.Context,
    source_identifier: int = typer.Argument(..., help="Source identifier", callback=util.sanitize),
) -> None:
    """Get a source."""

    @ensure_login
    def _request(ctx: typer.Context) -> httpx.Response:
        return util.get(
            ctx,
            url=f"{_source_url(ctx)}/{source_identifier}",
        )

    r = _request(ctx)
    process_response(r)


@app.command()
def update_source(
    ctx: typer.Context,
    source_identifier: int = typer.Argument(..., help="Source identifier", callback=util.sanitize),
    filepath: str = typer.Option(
        ...,
        "--filepath",
        "-f",
        help="Filepath of the source json payload",
        callback=util.sanitize,
    ),
) -> None:
    """Update a source."""

    @ensure_login
    def _request(ctx: typer.Context, content: schema.CreateSource) -> httpx.Response:
        return util.put(
            ctx,
            url=f"{_source_url(ctx)}/{source_identifier}",
            json=content.dict(exclude_none=True, by_alias=True),
        )

    fp = util.get_file_location(filepath)
    file_content: dict = util.load_json_file(fp, "json")  # type: ignore[reportGeneralTypeIssues]
    payload = schema.CreateSource(**file_content)
    r = _request(ctx, payload)
    process_response(r)


@app.command()
def delete_source(
    ctx: typer.Context,
    source_identifier: int = typer.Argument(..., help="Source identifier", callback=util.sanitize),
) -> None:
    """Delete a source."""

    @ensure_login
    def _request(ctx: typer.Context) -> httpx.Response:
        return util.delete(
            ctx,
            url=f"{_source_url(ctx)}/{source_identifier}",
        )

    r = _request(ctx)
    process_response(r)


@app.command()
def create_connection(
    ctx: typer.Context,
    name: str = typer.Option(..., "--name", "-n", help="Connection name", callback=util.sanitize),
    filepath: str = typer.Option(
        ...,
        "--filepath",
        "-f",
        help="Filepath of the connection json payload",
        callback=util.sanitize,
    ),
) -> None:
    """Create a new connection."""

    @ensure_login
    def _request(ctx: typer.Context, content: schema.CreateConnection) -> httpx.Response:
        return util.post(
            ctx,
            url=_connection_url(ctx),
            json=content.dict(exclude_none=True, by_alias=True),
        )

    fp = util.get_file_location(filepath)
    details: dict = util.load_object_file(fp, "connection")
    payload = schema.CreateConnection(
        name=name,
        details=details,  # type: ignore[reportGeneralTypeIssues]
    )
    r = _request(ctx, payload)
    process_response(r)


@app.command()
def list_connections(
    ctx: typer.Context,
) -> None:
    """List connections."""

    @ensure_login
    def _request(ctx: typer.Context) -> httpx.Response:
        return util.get(
            ctx,
            url=_connection_url(ctx),
        )

    r = _request(ctx)
    process_response(r)


@app.command()
def get_connection(
    ctx: typer.Context,
    connection_identifier: int = typer.Argument(..., help="Connection identifier", callback=util.sanitize),
) -> None:
    """Get a connection."""

    @ensure_login
    def _request(ctx: typer.Context) -> httpx.Response:
        return util.get(
            ctx,
            url=f"{_connection_url(ctx)}/{connection_identifier}",
        )

    r = _request(ctx)
    process_response(r)


@app.command()
def update_connection(
    ctx: typer.Context,
    connection_identifier: int = typer.Argument(..., help="Connection identifier", callback=util.sanitize),
    filepath: str = typer.Option(
        ...,
        "--filepath",
        "-f",
        help="Filepath of the connection json payload",
        callback=util.sanitize,
    ),
) -> None:
    """Update a connection."""

    @ensure_login
    def _request(ctx: typer.Context, content: schema.CreateConnection) -> httpx.Response:
        return util.put(
            ctx,
            url=f"{_connection_url(ctx)}/{connection_identifier}",
            json=content.dict(exclude_none=True, by_alias=True),
        )

    fp = util.get_file_location(filepath)
    file_content: dict = util.load_json_file(fp, "json")  # type: ignore[reportGeneralTypeIssues]
    payload = schema.CreateConnection(**file_content)
    r = _request(ctx, payload)
    process_response(r)


@app.command()
def delete_connection(
    ctx: typer.Context,
    connection_identifier: int = typer.Argument(..., help="Connection identifier", callback=util.sanitize),
) -> None:
    """Delete a connection."""

    @ensure_login
    def _request(ctx: typer.Context) -> httpx.Response:
        return util.delete(
            ctx,
            url=f"{_connection_url(ctx)}/{connection_identifier}",
        )

    r = _request(ctx)
    process_response(r)


@app.command()
def create_source_connection(
    ctx: typer.Context,
    source_identifier: int = typer.Option(
        ...,
        "--source",
        "-s",
        help="Source identifier",
        callback=util.sanitize,
    ),
    connection_identifier: int = typer.Option(
        ...,
        "--connection",
        "-c",
        help="Connection identifier",
        callback=util.sanitize,
    ),
    filepath: Optional[str] = typer.Option(
        None,
        "--filepath",
        "-f",
        help="Filepath of the connected source json payload",
        callback=util.sanitize,
    ),
    raw_json: Optional[str] = typer.Option(
        None,
        "--json",
        "-j",
        help="Connected source json payload",
        callback=util.sanitize,
    ),
) -> None:
    """Create a linkage between source and connection."""

    @ensure_login
    def _request(ctx: typer.Context, content: schema.CreateSourceConnection) -> httpx.Response:
        return util.post(
            ctx,
            url=_source_connection_url(ctx, source_id=source_identifier, connection_id=connection_identifier),
            json=content.dict(exclude_none=True, by_alias=True),
        )

    if filepath:
        fp = util.get_file_location(filepath)
        details: dict = util.load_object_file(fp, "source_connection")  # type: ignore[reportGeneralTypeIssues]
    else:
        details: dict = json.loads(raw_json)  # type: ignore[reportGeneralTypeIssues]

    payload = schema.CreateSourceConnection(details=details)  # type: ignore[reportGeneralTypeIssues]
    r = _request(ctx, payload)
    process_response(r)


@app.command()
def update_source_connection(
    ctx: typer.Context,
    source_identifier: int = typer.Option(
        ...,
        "--source",
        "-s",
        help="Source identifier",
        callback=util.sanitize,
    ),
    connection_identifier: int = typer.Option(
        ...,
        "--connection",
        "-c",
        help="Connection identifier",
        callback=util.sanitize,
    ),
    filepath: Optional[str] = typer.Option(
        None,
        "--filepath",
        "-f",
        help="Filepath of the connected source json payload",
        callback=util.sanitize,
    ),
    raw_json: Optional[str] = typer.Option(
        None,
        "--json",
        "-j",
        help="Connected source json payload",
        callback=util.sanitize,
    ),
) -> None:
    """Update a linkage between source and connection."""

    @ensure_login
    def _request(ctx: typer.Context, content: schema.CreateSourceConnection) -> httpx.Response:
        return util.put(
            ctx,
            url=_source_connection_url(ctx, source_id=source_identifier, connection_id=connection_identifier),
            json=content.details.dict(exclude_none=True, by_alias=True),
        )

    if filepath:
        fp = util.get_file_location(filepath)
        details: dict = util.load_object_file(fp, "source_connection")  # type: ignore[reportGeneralTypeIssues]
    else:
        details: dict = json.loads(raw_json)  # type: ignore[reportGeneralTypeIssues]

    payload = schema.CreateSourceConnection(details=details)  # type: ignore[reportGeneralTypeIssues]
    r = _request(ctx, payload)
    process_response(r)


@app.command()
def delete_source_connection(
    ctx: typer.Context,
    source_identifier: int = typer.Option(
        ...,
        "--source",
        "-s",
        help="Source identifier",
        callback=util.sanitize,
    ),
    connection_identifier: int = typer.Option(
        ...,
        "--connection",
        "-c",
        help="Connection identifier",
        callback=util.sanitize,
    ),
) -> None:
    """Delete a linkage between source and and connection."""

    @ensure_login
    def _request(ctx: typer.Context) -> httpx.Response:
        return util.delete(
            ctx,
            url=f"{_source_connection_url(ctx, source_id=source_identifier, connection_id=connection_identifier)}",
        )

    r = _request(ctx)
    process_response(r)



import sys
import typing

import httpx

if sys.platform != "win32":
    import jq
else:
    jq = None

import typer

from neosctl import schema, util
from neosctl.auth import ensure_login
from neosctl.util import process_response

app = typer.Typer()


def _product_url(ctx: typer.Context) -> str:
    return "{}/product".format(ctx.obj.get_gateway_api_url().rstrip("/"))


@app.command(name="create")
def create(
    ctx: typer.Context,
    name: str = typer.Argument(..., help="Data Product name", callback=util.sanitize),
    description: typing.Optional[str] = typer.Option(
        None,
        "--description",
        "-d",
        help="Data Product description",
        callback=util.sanitize,
    ),
    filepath: str = typer.Option(
        ...,
        "--filepath",
        "-f",
        help="Filepath of the details json payload",
        callback=util.sanitize,
    ),
) -> None:
    """Create a data product."""

    @ensure_login
    def _request(ctx: typer.Context, dpc: schema.CreateDataProduct) -> httpx.Response:
        return util.post(
            ctx,
            f"{_product_url(ctx)}",
            json=dpc.dict(exclude_none=True, by_alias=True),
        )

    fp = util.get_file_location(filepath)
    details = util.load_object_file(fp, "details")

    dpc = schema.CreateDataProduct(
        name=name,
        description=description or "",
        details=details,  # type: ignore[reportGeneralTypeIssues]
    )

    r = _request(ctx, dpc)
    process_response(r)


@app.command(name="add-schema")
def add_schema(
    ctx: typer.Context,
    product_identifier: str = typer.Argument(..., help="Data Product identifier", callback=util.sanitize),
    filepath: str = typer.Option(
        ...,
        "--filepath",
        "-f",
        help="Filepath of the table schema json payload",
        callback=util.sanitize,
    ),
) -> None:
    """Add a schema to a stored data product."""

    @ensure_login
    def _request(ctx: typer.Context, dps: schema.UpdateDataProductSchema) -> httpx.Response:
        return util.post(
            ctx,
            f"{_product_url(ctx)}/{product_identifier}/schema",
            json=dps.dict(exclude_none=True, by_alias=True),
        )

    fp = util.get_file_location(filepath)
    details = util.load_object_file(fp, "details")

    dps = schema.UpdateDataProductSchema(details=details)  # type: ignore[reportGeneralTypeIssues]

    r = _request(ctx, dps)
    process_response(r)


@app.command(name="list")
def list_products(
    ctx: typer.Context,
    filter_: typing.Optional[str] = typer.Option(
        None,
        "--filter",
        "-f",
        help="jq filter to apply to data_products.",
        callback=util.sanitize,
    ),
    name: typing.Optional[str] = typer.Option(
        None,
        "--name",
        "-n",
        help="Filter products for a specific name. (Overrides --filter if provided)",
        callback=util.sanitize,
    ),
) -> None:
    """List data products.

    Provide a filter to access specific elements in the returned list.
    """

    @ensure_login
    def _request(ctx: typer.Context) -> httpx.Response:
        return util.get(ctx, _product_url(ctx))

    if jq is None and (name or filter_):
        raise util.exit_with_output(
            msg=f"--filter and --name are not supported on '{sys.platform}'",
            exit_code=1,
        )
    if name:
        filter_ = f'{{data_products: [.data_products[] | select(.name | contains("{name}"))]}}'

    def _filter_products(payload: typing.Dict) -> str:
        """Filter response with an optional jq filter."""
        if filter_:
            output = jq.compile(filter_).input(payload)
            payload = output.all()
            if name:
                payload = output.first()

        return util.prettify_json(payload)

    r = _request(ctx)

    process_response(r, _filter_products)  # type: ignore[reportGeneralTypeIssues]


@app.command()
def delete_data(
    ctx: typer.Context,
    product_identifier: str = typer.Argument(..., help="Data Product identifier", callback=util.sanitize),
) -> None:
    """Delete data from a data product."""

    @ensure_login
    def _request(ctx: typer.Context) -> httpx.Response:
        return util.delete(
            ctx,
            f"{_product_url(ctx)}/{product_identifier}/data",
        )

    r = _request(ctx)
    process_response(r)


@app.command()
def delete(
    ctx: typer.Context,
    product_identifier: str = typer.Argument(..., help="Data Product identifier", callback=util.sanitize),
    *,
    force: bool = typer.Option(
        False,
        "--force",
        help="Force remove even if attached spark application is still running.",
    ),
) -> None:
    """Delete a data product."""

    @ensure_login
    def _request(ctx: typer.Context) -> httpx.Response:
        return util.delete(
            ctx,
            f"{_product_url(ctx)}/{product_identifier}",
            params={"force": force},
        )

    r = _request(ctx)
    process_response(r)


@app.command()
def publish(
    ctx: typer.Context,
    product_identifier: str = typer.Argument(..., help="Data Product identifier", callback=util.sanitize),
) -> None:
    """Publish a data product."""

    @ensure_login
    def _request(ctx: typer.Context) -> httpx.Response:
        return util.post(
            ctx,
            f"{_product_url(ctx)}/{product_identifier}/publish",
        )

    r = _request(ctx)
    process_response(r)


@app.command()
def unpublish(
    ctx: typer.Context,
    product_identifier: str = typer.Argument(..., help="Data Product identifier", callback=util.sanitize),
) -> None:
    """Unpublish a product."""

    @ensure_login
    def _request(ctx: typer.Context) -> httpx.Response:
        return util.delete(
            ctx,
            f"{_product_url(ctx)}/{product_identifier}/publish",
        )

    r = _request(ctx)
    process_response(r)


@app.command(name="get")
def get_product(
    ctx: typer.Context,
    product_identifier: str = typer.Argument(
        ...,
        help="Data Product identifier",
        callback=util.sanitize,
    ),
) -> None:
    """Get data product schema."""

    @ensure_login
    def _request(ctx: typer.Context) -> httpx.Response:
        return util.get(
            ctx,
            f"{_product_url(ctx)}/{product_identifier}",
        )

    r = _request(ctx)
    process_response(r)


@app.command()
def preview(
    ctx: typer.Context,
    product_identifier: str = typer.Argument(
        ...,
        help="Data Product identifier",
        callback=util.sanitize,
    ),
) -> None:
    """Preview data product data.

    Get the first 25 rows of a data product's data.
    """

    @ensure_login
    def _request(ctx: typer.Context) -> httpx.Response:
        return util.get(
            ctx,
            "{product_url}/{name}/data".format(
                product_url=_product_url(ctx),
                name=product_identifier,
            ),
        )

    r = _request(ctx)
    process_response(r)



import typing

import httpx
import typer

from neosctl import util
from neosctl.auth import ensure_login
from neosctl.util import process_response

app = typer.Typer()


def _kafka_topic_url(data_product: str, gateway_api_url: str) -> str:
    return "{}/kafka/{}/topic".format(gateway_api_url.rstrip("/"), data_product)


def _kafka_url(data_product: str, gateway_api_url: str) -> str:
    return "{}/kafka/{}".format(gateway_api_url.rstrip("/"), data_product)


@app.command()
def create_topic(
    ctx: typer.Context,
    product_identifier: str = typer.Argument(
        ...,
        help="Data Product identifier",
        callback=util.sanitize,
    ),
    num_partitions: int = typer.Option(
        1,
        "--partitions",
        help="Number of partitions to create.",
    ),
    replication_factor: typing.Optional[int] = typer.Option(
        None,
        "--replication-factor",
        help="Replication factor of partitions, or -1 if replica_assignment is used.",
    ),
    replica_assignment: typing.Optional[str] = typer.Option(
        None,
        "--replica-assignment",
        help="""<bid_p1_r1:bid_p1_r2,bid_p2_r1:bid_p2_r2> A string representing list of manual partition to broker
        assignments for topic being created.
        Replication assignment (list of lists). The outer list index represents the partition index,
        the inner list is the replica assignment (broker ids) for that partition.
        replication_factor and replica_assignment are mutually exclusive.""",
        callback=util.sanitize,
    ),
    config: typing.Optional[typing.List[str]] = typer.Option(
        None,
        "--config",
        help="""<name=value> An override of the topic configuration being created.
        See http://kafka.apache.org/documentation.html#topicconfigs.""",
        callback=util.sanitize,
    ),
) -> None:
    """Create kafka topic for data product."""

    @ensure_login
    def _request(ctx: typer.Context, topic_settings: typing.Dict) -> httpx.Response:
        return util.post(
            ctx,
            _kafka_topic_url(product_identifier, ctx.obj.gateway_api_url),
            json=topic_settings,
        )

    topic_settings: typing.Dict[str, typing.Any] = {
        "num_partitions": num_partitions,
    }
    if replication_factor:
        topic_settings["replication_factor"] = replication_factor

    if replica_assignment:
        replica_assignments = []
        for assignment in replica_assignment.split(","):
            broker_ids = [int(a) for a in assignment.split(":")]
            replica_assignments.append(broker_ids)
        topic_settings["replica_assignment"] = replica_assignments

    if config:
        overrides = {}
        for override in config:
            name, value = override.split("=")
            overrides[name] = value
        topic_settings["config"] = overrides

    r = _request(ctx, topic_settings)

    process_response(r)


@app.command()
def list_topic(
    ctx: typer.Context,
    product_identifier: str = typer.Argument(
        ...,
        help="Data Product identifier",
        callback=util.sanitize,
    ),
) -> None:
    """List kafka topic from data product."""

    @ensure_login
    def _request(ctx: typer.Context) -> httpx.Response:
        return util.get(
            ctx,
            _kafka_topic_url(product_identifier, ctx.obj.gateway_api_url),
        )

    r = _request(ctx)

    process_response(r)


@app.command()
def delete_topic(
    ctx: typer.Context,
    product_identifier: str = typer.Argument(
        ...,
        help="Data Product identifier",
        callback=util.sanitize,
    ),
) -> None:
    """Delete kafka topic from data product."""

    @ensure_login
    def _request(ctx: typer.Context) -> httpx.Response:
        return util.delete(
            ctx,
            _kafka_topic_url(product_identifier, ctx.obj.gateway_api_url),
        )

    r = _request(ctx)

    process_response(r)


@app.command()
def produce(
    ctx: typer.Context,
    product_identifier: str = typer.Argument(
        ...,
        help="Data Product identifier",
        callback=util.sanitize,
    ),
    filepath: str = typer.Option(
        ...,
        "--filepath",
        "-f",
        help="Filepath of the records json payload",
        callback=util.sanitize,
    ),
) -> None:
    """Send one or more records to a given topic, optionally specifying a partition, key, or both.

    JSON example

    \b
    {
      "records": [
        {
          "key": "string",
          "value": "string",
          "partition": 0
        }
      ]
    }
    """  # noqa: D301

    @ensure_login
    def _request(ctx: typer.Context, data: typing.Dict) -> httpx.Response:
        return util.post(
            ctx,
            _kafka_url(product_identifier, ctx.obj.gateway_api_url),
            json=data,
        )

    fp = util.get_file_location(filepath)
    data = util.load_object_file(fp, "records")

    r = _request(ctx, data)

    process_response(r)



import sys
import typing

if sys.version_info < (3, 10):
    from typing_extensions import ParamSpec
else:
    from typing import ParamSpec


import click
import httpx
import typer

from neosctl import schema, util
from neosctl.util import (
    check_profile_exists,
    constant,
    exit_with_output,
    get_user_profile,
    is_success_response,
    process_response,
    read_config_dotfile,
    upsert_config,
)

app = typer.Typer()


def _auth_url(iam_api_url: str) -> str:
    return "{}".format(iam_api_url.rstrip("/"))


def _check_refresh_token_exists(ctx: click.Context) -> bool:
    if ctx.obj.profile.refresh_token == "":  # nosec: B105
        raise exit_with_output(
            msg=f"You need to login. Run neosctl -p {ctx.obj.profile_name} auth login",
            exit_code=1,
        )

    return True


def _refresh_token(ctx: click.Context) -> httpx.Response:
    check_profile_exists(ctx)
    _check_refresh_token_exists(ctx)

    r = util.post(
        ctx,
        f"{_auth_url(ctx.obj.get_iam_api_url())}/refresh",
        json={"refresh_token": ctx.obj.profile.refresh_token},
    )

    if not is_success_response(r):
        process_response(r)

    upsert_config(ctx, _update_profile(ctx, schema.Auth(**r.json())))

    return r


P = ParamSpec("P")


def ensure_login(method: typing.Callable[P, httpx.Response]) -> typing.Callable[P, httpx.Response]:
    """Capture authentication errors and retry requests.

    On request failure check if the response is a 401, and attempt to refresh the access_token.
    Retry the request with refreshed token, on subsequent failure, return.
    """

    def check_access_token(*args: P.args, **kwargs: P.kwargs) -> httpx.Response:
        ctx = args[0]
        if not isinstance(ctx, click.Context):
            # Developer reminder
            msg = "First argument should be click.Context instance"
            raise TypeError(msg)

        r = method(*args, **kwargs)

        check_profile_exists(ctx)

        # Try to refresh token
        # Confirm it is a token invalid 401, registry not configured mistriggers this flow.
        if r.status_code == constant.UNAUTHORISED_CODE:
            data = r.json()
            if "code" in data and data["code"].startswith("A0"):
                _refresh_token(ctx)

                # Refresh the context
                c = read_config_dotfile()
                ctx.obj.config = c
                ctx.obj.profile = get_user_profile(c, ctx.obj.profile_name)

                r = method(*args, **kwargs)

        return r  # noqa: RET504

    return check_access_token


def _update_profile(
    ctx: click.Context,
    auth: schema.Auth = schema.Auth(),
) -> schema.Profile:
    return schema.Profile(
        gateway_api_url=ctx.obj.gateway_api_url,
        registry_api_url=ctx.obj.registry_api_url,
        iam_api_url=ctx.obj.iam_api_url,
        storage_api_url=ctx.obj.storage_api_url,
        user=ctx.obj.profile.user,
        access_token=auth.access_token,
        refresh_token=auth.refresh_token,
        ignore_tls=ctx.obj.profile.ignore_tls,
    )


@app.command()
def login(
    ctx: typer.Context,
    password: typing.Optional[str] = typer.Option(None, "--password", "-p", callback=util.sanitize),
) -> None:
    """Login to neos."""
    check_profile_exists(ctx)

    if password is None:
        password = typer.prompt(
            "[{profile}] Enter password for user ({user})".format(
                profile=ctx.obj.profile_name,
                user=ctx.obj.profile.user,
            ),
            hide_input=True,
        )

    r = util.post(
        ctx,
        f"{_auth_url(ctx.obj.get_iam_api_url())}/login",
        json={"user": ctx.obj.profile.user, "password": password},
    )

    if not is_success_response(r):
        process_response(r)

    upsert_config(ctx, _update_profile(ctx, schema.Auth(**r.json())))

    raise exit_with_output(
        msg="Login success",
        exit_code=0,
    )


@app.command()
def logout(ctx: typer.Context) -> None:
    """Logout from neos."""
    check_profile_exists(ctx)

    _check_refresh_token_exists(ctx)

    r = util.post(
        ctx,
        f"{_auth_url(ctx.obj.get_iam_api_url())}/logout",
        json={"refresh_token": ctx.obj.profile.refresh_token},
    )

    if not is_success_response(r):
        process_response(r)

    upsert_config(ctx, _update_profile(ctx, schema.Auth()))

    raise exit_with_output(
        msg="Logout success",
        exit_code=0,
    )



import httpx
import typer

from neosctl import util
from neosctl.auth import ensure_login
from neosctl.util import process_response

app = typer.Typer()


def _consume_url(ctx: typer.Context) -> str:
    return "{}/consume".format(ctx.obj.gateway_api_url.rstrip("/"))


@app.command()
def query(
    ctx: typer.Context,
    statement: str = typer.Argument(..., callback=util.sanitize),
) -> None:
    """Query a published data product."""

    @ensure_login
    def _request(ctx: typer.Context) -> httpx.Response:
        return util.post(
            ctx,
            url=_consume_url(ctx),
            json={
                "statement": statement,
            },
        )

    r = _request(ctx)
    process_response(r)



import configparser
import os
import typing

import typer

import neosctl
from neosctl import (
    auth,
    constant,
    consume,
    iam,
    kafka,
    metadata,
    product,
    profile,
    registry,
    schema,
    secret,
    source,
    spark,
    storage,
    util,
)
from neosctl.util import get_user_profile, read_config_dotfile


def _generate_common_schema(
    gateway_api_url: str,
    registry_api_url: str,
    iam_api_url: str,
    storage_api_url: str,
    profile_name: str,
    config: configparser.ConfigParser,
    profile: typing.Union[schema.Profile, schema.OptionalProfile, None] = None,
) -> schema.Common:
    """Generate a common schema object.

    Set the default api values using user_profile and cli provided defaults.
    """
    common_schema = schema.Common(
        gateway_api_url=gateway_api_url,
        registry_api_url=registry_api_url,
        iam_api_url=iam_api_url,
        storage_api_url=storage_api_url,
        profile_name=profile_name,
        config=config,
        profile=profile,
    )
    common_schema.gateway_api_url = common_schema.get_gateway_api_url()
    common_schema.registry_api_url = common_schema.get_registry_api_url()
    common_schema.iam_api_url = common_schema.get_iam_api_url()
    common_schema.storage_api_url = common_schema.get_storage_api_url()

    return common_schema


def _version_callback(*, value: bool) -> None:
    """Get current cli version."""
    if value:
        typer.echo(f"neosctl {neosctl.__version__}")
        raise typer.Exit


def _callback(
    ctx: typer.Context,
    version: typing.Optional[bool] = typer.Option(  # noqa: ARG001
        None,
        "--version",
        callback=_version_callback,
        help="Print version and exit.",
    ),
    gateway_api_url: str = typer.Option(
        "",
        "--gateway-api-url",
        "--gurl",
        help="Gateway API URL",
        callback=util.sanitize,
    ),
    registry_api_url: str = typer.Option(
        "",
        "--registry-api-url",
        "--rurl",
        help="Registry API URL",
        callback=util.sanitize,
    ),
    iam_api_url: str = typer.Option("", "--iam-api-url", "--iurl", help="IAM API URL", callback=util.sanitize),
    storage_api_url: str = typer.Option(
        "",
        "--storage-api-url",
        "--surl",
        help="Storage API URL",
        callback=util.sanitize,
    ),
    profile: str = typer.Option(
        os.getenv("NEOSCTL_PROFILE", constant.DEFAULT_PROFILE),
        "--profile",
        "-p",
        help="Profile name",
        callback=util.sanitize,
    ),
) -> None:
    """Inject common configuration defaults into context.

    Allow missing user_profile to support profile generation etc.
    """
    config = read_config_dotfile()
    user_profile = get_user_profile(config, profile, allow_missing=True)

    common_schema = _generate_common_schema(
        gateway_api_url=gateway_api_url,
        registry_api_url=registry_api_url,
        iam_api_url=iam_api_url,
        storage_api_url=storage_api_url,
        profile_name=profile,
        config=config,
        profile=user_profile,
    )

    ctx.obj = common_schema


def _common(
    ctx: typer.Context,
) -> None:
    """Inject required user_profile into context."""
    user_profile = get_user_profile(ctx.obj.config, ctx.obj.profile_name)
    ctx.obj.profile = user_profile


app = typer.Typer(name="neosctl", callback=_callback)
app.add_typer(profile.app, name="profile", help="Manage profiles.")
app.add_typer(auth.app, name="auth", callback=_common, help="Manage authentication status.")
app.add_typer(consume.app, name="consume", callback=_common, help="Consume published data products.")
app.add_typer(iam.app, name="iam", callback=_common, help="Manage access policies.")
app.add_typer(storage.app, name="storage", callback=_common, help="Interact with Storage (as a service).")
app.add_typer(metadata.app, name="metadata", callback=_common, help="Manage and browse metadata.")
app.add_typer(product.app, name="product", callback=_common, help="Manage data products.")
app.add_typer(source.app, name="source", help="Manage sources.")
app.add_typer(registry.app, name="registry", callback=_common, help="Manage cores and search data products.")
app.add_typer(spark.app, name="spark", callback=_common, help="Manage data product spark jobs.")
app.add_typer(secret.app, name="secret", callback=_common, help="Manage secrets.")
app.add_typer(kafka.app, name="kafka", callback=_common, help="Manage data product kafka streaming.")
